"""A bot to controll settings for tgcf live mode."""

import yaml
from telethon import events

from tgcf import config, const, plugins
from tgcf.bot.utils import admin_protect, display_forwards, get_args, remove_source


@admin_protect
async def forward_command_handler(event):
    """Handle the `/forward` command."""
    notes = """The `/forward` command allows you to add a new forward.
    Example: suppose you want to forward from a to (b and c)

    ```
    /forward source: a
    dest: [b,c]
    ```

    a,b,c are chat ids

    """.replace(
        "    ", ""
    )

    try:
        args = get_args(event.message.text)
        if not args:
            raise ValueError(
                f"{notes}\n{display_forwards(config.CONFIG.forwards)}")

        parsed_args = yaml.safe_load(args)
        print(parsed_args)
        forward = config.Forward(**parsed_args)
        print(forward)
        try:
            remove_source(forward.source, config.CONFIG.forwards)
        except:
            pass
        config.CONFIG.forwards.append(forward)
        config.from_to = config.load_from_to(config.CONFIG.forwards)

        await event.respond("Success")
        config.update_config_file(config.CONFIG)
    except ValueError as err:
        print(err)
        await event.respond(str(err))

    finally:
        raise events.StopPropagation


@admin_protect
async def remove_command_handler(event):
    """Handle the /remove command."""
    notes = """The `/remove` command allows you to remove a source from forwarding.
    Example: Suppose you want to remove the channel with id -100, then run

    `/remove source: -100`

    """.replace(
        "    ", ""
    )

    try:
        args = get_args(event.message.text)
        if not args:
            raise ValueError(
                f"{notes}\n{display_forwards(config.CONFIG.forwards)}")

        parsed_args = yaml.safe_load(args)
        print(parsed_args)
        source_to_remove = parsed_args.get("source")
        print(source_to_remove)
        config.CONFIG.forwards = remove_source(
            source_to_remove, config.CONFIG.forwards)
        print(config.CONFIG.forwards)
        config.from_to = config.load_from_to(config.CONFIG.forwards)

        await event.respond("Success")
        config.update_config_file(config.CONFIG)
    except ValueError as err:
        print(err)
        await event.respond(str(err))

    finally:
        raise events.StopPropagation


@admin_protect
async def style_command_handler(event):
    """Handle the /style command"""
    notes = """This command is used to set the style of the messages to be forwarded.

    Example: `/style bold`

    """.replace("    ", "")

    try:
        args = get_args(event.message.text)
        if not args:
            raise ValueError(
                f"{notes}\n")
        config.CONFIG.plugins.update({"format": {"style": args}})
        config.update_config_file(config.CONFIG)
        plugins.plugins = plugins.load_plugins()
        await event.respond("Success")

    except ValueError as err:
        print(err)
        await event.respond(str(err))

    finally:
        raise events.StopPropagation


async def start_command_handler(event):
    """Handle the /start command."""
    await event.respond(const.BotMessages.start)


async def help_command_handler(event):
    """Handle the /help command."""
    await event.respond(const.BotMessages.bot_help)


BOT_EVENTS = {
    "bot_start": (start_command_handler, events.NewMessage(pattern="/start")),
    "bot_forward": (forward_command_handler, events.NewMessage(pattern="/forward")),
    "bot_remove": (remove_command_handler, events.NewMessage(pattern="/remove")),
    "bot_style": (style_command_handler, events.NewMessage(pattern="/style")),
    "bot_help": (help_command_handler, events.NewMessage(pattern="/help")),
}



"""Load and apply plugins."""

import logging
from importlib import import_module
from typing import List

from tgcf.config import CONFIG

PLUGINS = CONFIG.plugins


def load_plugins():
    """Load the plugins specified in config."""
    _plugins = {}
    for plugin_id, plugin_data in PLUGINS.items():
        plugin_module_name = f"tgcf_{plugin_id}"
        plugin_class_name = f"Tgcf{plugin_id.title()}"
        try:
            plugin_module = import_module(plugin_module_name)
            plugin_class = getattr(plugin_module, plugin_class_name)
            plugin = plugin_class(plugin_data)
            assert plugin.id_ == plugin_id

        except ModuleNotFoundError:
            logging.error(f"Could not find plugin for {plugin_id}")
        except AttributeError:
            logging.error(f"Found plugin {plugin_id}, but failed to load.")
        else:
            print(f"Loaded plugin {plugin_id}")
            _plugins.update({plugin.id_: plugin})
    return _plugins


def apply_plugins(message) -> List:
    """Apply all loaded plugins to a message."""
    for _id,plugin in plugins.items():
        message = plugin.modify(message)
        logging.info(f"Applied plugin {_id}")
        if not message:
            return None
    return message


plugins = load_plugins()



import logging
from enum import Enum
from typing import Dict, Optional

from pydantic import BaseModel


class Style(str, Enum):
    BOLD = ("bold",)
    ITALICS = "italics"
    CODE = "code"
    STRIKE = "strike"
    PLAIN = "plain"
    PRESERVE = "preserve"


class Format(BaseModel):
    style: Style = Style.PRESERVE


STYLE_CODES = {"bold": "**", "italics": "__",
               "code": "`", "strike": "~~", "plain": ""}


class TgcfFormat:
    id_ = "format"

    def __init__(self, data):
        self.format = Format(**data)
        logging.info(self.format)

    def modify(self, message):
        if self.format.style is Style.PRESERVE:
            return message
        msg_text: str = message.raw_text
        if not msg_text:
            return message
        style = STYLE_CODES.get(self.format.style)
        message.text = f"{style}{msg_text}{style}"
        return message



"""This module implements the command line interface for tgcf."""

import asyncio
import logging
import os
import sys
from enum import Enum
from typing import Optional

import typer
from dotenv import load_dotenv

from tgcf import __version__

load_dotenv(".env")

FAKE = bool(os.getenv("FAKE"))
app = typer.Typer(add_completion=False)


class Mode(str, Enum):
    """tgcf works in two modes."""

    PAST = "past"
    LIVE = "live"


def version_callback(value: bool):
    """Show current version and exit."""
    if value:
        print(__version__)
        raise typer.Exit()


def verbosity_callback(value: bool):
    """Set logging level."""
    if value:
        level = logging.INFO
    else:
        level = logging.WARNING
    logging.basicConfig(level=level)
    logging.info("Verbosity turned on. \nThis is suitable for debugging.\n")


@app.command()
def main(
    mode: Mode = typer.Argument(
        ..., help="Choose the mode in which you want to run tgcf.", envvar="TGCF_MODE"
    ),
    verbose: Optional[bool] = typer.Option(  # pylint: disable=unused-argument
        None,
        "--loud",
        "-l",
        callback=verbosity_callback,
        envvar="LOUD",
        help="Increase output verbosity.",
    ),
    version: Optional[bool] = typer.Option(  # pylint: disable=unused-argument
        None,
        "--version",
        "-v",
        callback=version_callback,
        help="Show version and exit.",
    ),
):
    """Tgcf is a powerful tool for forwarding telegram messages from source to destination.

    Don't forget to star  https://github.com/aahnik/tgcf

    Telegram Channel https://telegram.me/tg_cf
    """
    if FAKE:
        print(f"mode = {mode}")
        sys.exit(1)

    if mode == Mode.PAST:
        from tgcf.past import forward_job  # pylint: disable=import-outside-toplevel

        asyncio.run(forward_job())
    else:
        from tgcf.live import start_sync  # pylint: disable=import-outside-toplevel

        start_sync()


# AAHNIK 2021



"""Load all user defined config and env vars."""

import logging
import os
import sys
from typing import Dict, List, Optional, Union

import yaml
from dotenv import load_dotenv
from pydantic import BaseModel, validator  # pylint: disable=no-name-in-module
from telethon.sessions import StringSession

from tgcf.const import CONFIG_ENV_VAR_NAME, CONFIG_FILE_NAME

load_dotenv()


class Forward(BaseModel):
    """Blueprint for the forward object."""

    # pylint: disable=too-few-public-methods
    source: Union[int, str]
    dest: List[Union[int, str]]
    offset: Optional[int] = 0
    end: Optional[int] = None


class LiveSettings(BaseModel):
    """Settings to configure how tgcf operates in live mode."""

    # pylint: disable=too-few-public-methods
    delete_sync: bool = False
    delete_on_edit: Optional[str] = None


class PastSettings(BaseModel):
    """Configuration for past mode."""

    # pylint: disable=too-few-public-methods
    delay: Optional[float] = 0

    @validator("delay")
    def validate_delay(cls, val):  # pylint: disable=no-self-use,no-self-argument
        """Check if the delay used by user is values. If not, use closest logical values."""
        if val not in range(0, 101):
            logging.warning("delay must be within 0 to 100 seconds")
            if val > 100:
                val = 100
            if val < 0:
                val = 0
        return val


class Config(BaseModel):
    """The blueprint for tgcf's whole config."""

    # pylint: disable=too-few-public-methods
    forwards: List[Forward] = []
    show_forwarded_from: bool = False
    live: LiveSettings = LiveSettings()
    past: PastSettings = PastSettings()
    admins: List[int] = []

    plugins: Optional[Dict] = {}


def detect_config_type() -> int:
    """Return 0 when no config found, 1 when tgcf.config.yml, 2 when env var, else terminate."""
    tutorial_link = "Learn more http://bit.ly/configure-tgcf"

    if CONFIG_FILE_NAME in os.listdir():
        logging.info(f"{CONFIG_FILE_NAME} detected.")
        return 1
    if os.getenv("TGCF_CONFIG"):
        logging.info(f"env var {CONFIG_ENV_VAR_NAME} detected.")
        if not ".env" in os.listdir():
            return 2

        logging.warning(
            f"If you can create files in your system,\
            you should use tgcf.config.yml and not .env to define configuration. {tutorial_link}"
        )
        sys.exit(1)
    else:
        return 0


CONFIG_TYPE = detect_config_type()


def read_config() -> Config:
    """Load the configuration defined by user."""
    if CONFIG_TYPE == 1:
        with open(CONFIG_FILE_NAME) as file:
            config_dict = yaml.full_load(file)
    elif CONFIG_TYPE == 2:
        config_env_var = os.getenv("TGCF_CONFIG")
        config_dict = yaml.full_load(config_env_var)
    else:
        return Config()

    try:
        if config_dict:
            config = Config(**config_dict)
        else:
            config = Config()
    except Exception as err:
        print(err)
        sys.exit(1)
    else:
        logging.info(config)
        return config


def update_config_file(config: Config):
    """Write changes in config back to file."""
    if CONFIG_TYPE == 1 or CONFIG_TYPE == 0:
        with open(CONFIG_FILE_NAME, "w") as file:
            yaml.dump(config.dict(), file)
    elif CONFIG_TYPE == 2:
        logging.warning("Could not update config! As env var is used")


def get_env_var(name: str, optional=False):
    """Fetch an env var."""
    var = os.getenv(name, "")

    while not var:
        if optional:
            return ""
        var = input(f"Enter {name}: ")
    return var


API_ID = get_env_var("API_ID")
API_HASH = get_env_var("API_HASH")
USERNAME = get_env_var("USERNAME", optional=True)
SESSION_STRING = get_env_var("SESSION_STRING", optional=True)
BOT_TOKEN = get_env_var("BOT_TOKEN", optional=True)

if SESSION_STRING:
    SESSION = StringSession(SESSION_STRING)
elif BOT_TOKEN:
    SESSION = "tgcf_bot"
else:
    SESSION = "tgcf_user"

CONFIG = read_config()


def load_from_to(forwards: List[Forward]):
    """Load a from -> to mapping."""
    from_to_dict = {}
    for forward in forwards:
        from_to_dict[forward.source] = forward.dest
    return from_to_dict


from_to = load_from_to(CONFIG.forwards)

logging.info("config.py got executed")



import logging
from typing import Dict, Optional

from pydantic import BaseModel


class Replace(BaseModel):
    text: Optional[Dict[str, str]] = {}


class TgcfReplace:
    id_ = "replace"

    def __init__(self, data):
        self.replace = Replace(**data)
        logging.info(self.replace)

    def modify(self, message):
        msg_text: str = message.text
        if not msg_text:
            return message
        for original, new in self.replace.text.items():
            msg_text = msg_text.replace(original, new)
        message.text = msg_text
        return message



"""helper functions for the bot."""
from typing import List

from telethon import events

from tgcf import config
from tgcf.config import Forward


def admin_protect(org_func):
    """Decorate to restrict non admins from accessing the bot."""
    if config.CONFIG.admins == []:
        return org_func

    async def wrapper_func(event):
        """Wrap the original function."""
        if event.sender_id not in config.CONFIG.admins:
            await event.respond("You are not authorized.")
            raise events.StopPropagation
        return await org_func(event)

    return wrapper_func


def get_args(text: str):
    """Return the part of message following the command."""
    splitted = text.split(" ", 1)

    if not len(splitted) == 2:
        splitted = text.split("\n",1)
        if not len(splitted) == 2:
            return ""

    prefix, args = splitted
    print(prefix)
    args = args.strip()
    print(args)
    return args


def display_forwards(forwards: List[Forward]):
    """Return a string that beautifully displays all current forwards."""
    if len(forwards) == 0:
        return "Currently no forwards are set"
    forward_str = "This is your configuration"
    for forward in forwards:
        forward_str = (
            forward_str
            + f"\n\n```\nsource: {forward.source}\ndest: {forward.dest}\n```\n"
        )

    return forward_str


def remove_source(source, forwards: List[Forward]):
    """Remove a source from forwards."""
    for i, forward in enumerate(forwards):
        print(forward)
        print(type(forward.source))
        print(type(source))
        if forward.source == source:
            del forwards[i]
            return forwards
    raise ValueError("The source does not exist")



"""The module for running tgcf in past mode.

- past mode can only operate with a user account.
- past mode deals with all existing messages.
"""

import asyncio
import logging
import time

from telethon import TelegramClient
from telethon.errors.rpcerrorlist import FloodWaitError
from telethon.tl.patched import MessageService

from tgcf.config import API_HASH, API_ID, CONFIG, SESSION, update_config_file
from tgcf.plugins import apply_plugins
from tgcf.utils import send_message


async def forward_job():
    """Forward all existing messages in the concerned chats."""
    async with TelegramClient(SESSION, API_ID, API_HASH) as client:
        for forward in CONFIG.forwards:
            last_id = 0
            logging.info(f"Forwarding messages from {forward.source} to {forward.dest}")
            async for message in client.iter_messages(
                forward.source, reverse=True, offset_id=forward.offset
            ):
                if isinstance(message, MessageService):
                    continue
                try:
                    message = apply_plugins(message)
                    if not message:
                        continue

                    for destination in forward.dest:
                        await send_message(client, destination, message)
                    last_id = str(message.id)
                    logging.info(f"forwarding message with id = {last_id}")
                    forward.offset = last_id
                    update_config_file(CONFIG)
                    time.sleep(CONFIG.past.delay)
                    logging.info(f"slept for {CONFIG.past.delay} seconds")

                except FloodWaitError as fwe:
                    print(f"Sleeping for {fwe}")
                    await asyncio.sleep(delay=fwe.seconds)
                except Exception as err:
                    logging.exception(err)



"""The module responsible for operating tgcf in live mode."""
import logging

from telethon import events

from tgcf import config, const
from tgcf.bot import BOT_EVENTS
from tgcf.plugins import apply_plugins, plugins
from tgcf.utils import send_message

existing_hashes = []

_stored = {}


class EventUid:
    """The objects of this class uniquely identifies an event."""

    def __init__(self, event):
        self.chat_id = event.chat_id
        try:
            self.msg_id = event.id
        except:  # pylint: disable=bare-except
            self.msg_id = event.deleted_id

    def __str__(self):
        return f"chat={self.chat_id} msg={self.msg_id}"

    def __eq__(self, other):
        return self.chat_id == other.chat_id and self.msg_id == other.msg_id

    def __hash__(self) -> int:
        return hash(self.__str__())


async def new_message_handler(event):
    """Process new incoming messages."""
    chat_id = event.chat_id

    if chat_id not in config.from_to:
        return
    logging.info(f"New message received in {chat_id}")
    message = event.message

    global _stored  # pylint: disable=global-statement,invalid-name

    event_uid = EventUid(event)

    length = len(_stored)
    exceeding = length - const.KEEP_LAST_MANY

    if exceeding > 0:
        for key in _stored:
            del _stored[key]
            break

    to_send_to = config.from_to.get(chat_id)

    if to_send_to:
        if event_uid not in _stored:
            _stored[event_uid] = []

        message = apply_plugins(message)
        if not message:
            return
        for recipient in to_send_to:
            fwded_msg = await send_message(event.client, recipient, message)
            _stored[event_uid].append(fwded_msg)

    existing_hashes.append(hash(message.text))


async def edited_message_handler(event):
    """Handle message edits."""
    message = event.message

    chat_id = event.chat_id

    if chat_id not in config.from_to:
        return

    logging.info(f"Message edited in {chat_id}")

    event_uid = EventUid(event)

    message = apply_plugins(message)

    if not message:
        return

    fwded_msgs = _stored.get(event_uid)

    if fwded_msgs:
        for msg in fwded_msgs:
            if config.CONFIG.live.delete_on_edit == message.raw_text:
                await msg.delete()
                await message.delete()
            else:
                await msg.edit(message.text)
        return

    to_send_to = config.from_to.get(event.chat_id)

    for recipient in to_send_to:
        await send_message(event.client, recipient, message)


async def deleted_message_handler(event):
    """Handle message deletes."""
    chat_id = event.chat_id
    if chat_id not in config.from_to:
        return

    logging.info(f"Message deleted in {chat_id}")

    event_uid = EventUid(event)
    fwded_msgs = _stored.get(event_uid)
    if fwded_msgs:
        for msg in fwded_msgs:
            await msg.delete()
        return


ALL_EVENTS = {
    "new": (new_message_handler, events.NewMessage()),
    "edited": (edited_message_handler, events.MessageEdited()),
    "deleted": (deleted_message_handler, events.MessageDeleted()),
}

ALL_EVENTS.update(BOT_EVENTS)

for _, plugin in plugins.items():
    try:
        event_handlers = getattr(plugin, "event_handlers")
        if event_handlers:
            ALL_EVENTS.update(event_handlers)
    except AttributeError:
        pass


def start_sync():
    """Start tgcf live sync."""
    # pylint: disable=import-outside-toplevel
    from telethon.sync import TelegramClient, functions, types

    client = TelegramClient(config.SESSION, config.API_ID, config.API_HASH)
    client.start(bot_token=config.BOT_TOKEN)
    is_bot = client.is_bot()

    for key, val in ALL_EVENTS.items():
        if key.startswith("bot"):
            if not is_bot:
                continue
        if config.CONFIG.live.delete_sync is False and key == "deleted":
            continue
        client.add_event_handler(*val)
        logging.info(f"Added event handler for {key}")

    if is_bot and const.REGISTER_COMMANDS:
        client(
            functions.bots.SetBotCommandsRequest(
                commands=[
                    types.BotCommand(command=key, description=value)
                    for key, value in const.COMMANDS.items()
                ]
            )
        )

    client.run_until_disconnected()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'plugins'}

packages = \
['tgcf',
 'tgcf.bot',
 'tgcf_filter',
 'tgcf_format',
 'tgcf_ocr',
 'tgcf_replace',
 'tgcf_watermark']

package_data = \
{'': ['*']}

install_requires = \
['Pillow>=8.1.2,<9.0.0',
 'PyYAML>=5.4.1,<6.0.0',
 'Telethon>=1.20,<2.0',
 'aiohttp>=3.7.4,<4.0.0',
 'cryptg>=0.2.post2,<0.3',
 'hachoir>=3.1.2,<4.0.0',
 'pydantic>=1.8.1,<2.0.0',
 'python-dotenv>=0.17.0,<0.18.0',
 'requests>=2.25.1,<3.0.0',
 'tg-login>=0.0.2,<0.0.3',
 'typer>=0.3.2,<0.4.0']

entry_points = \
{'console_scripts': ['tgcf = tgcf.cli:app']}

setup_kwargs = {
    'name': 'tgcf',
    'version': '0.1.35.post6',
    'description': 'The ultimate tool to automate custom telegram message forwarding.',
    'long_description': '<!-- markdownlint-disable -->\n\n<p align="center">\n<a href = "https://github.com/aahnik/tgcf" > <img src = "https://user-images.githubusercontent.com/66209958/115183360-3fa4d500-a0f9-11eb-9c0f-c5ed03a9ae17.png" alt = "tgcf logo"  width=120> </a>\n</p>\n\n<h1 align="center"> tgcf </h1>\n\n<p align="center">\nThe ultimate tool to automate telegram message forwarding.\n</p>\n\n<p align="center"><a href="https://github.com/aahnik/tgcf/blob/main/LICENSE"><img src="https://img.shields.io/github/license/aahnik/tgcf" alt="GitHub license"></a>\n<a href="https://github.com/aahnik/tgcf/stargazers"><img src="https://img.shields.io/github/stars/aahnik/tgcf?style=social" alt="GitHub stars"></a>\n<a href="https://github.com/aahnik/tgcf/issues"><img src="https://img.shields.io/github/issues/aahnik/tgcf" alt="GitHub issues"></a>\n<img src="https://img.shields.io/pypi/v/tgcf" alt="PyPI">\n<a href="https://twitter.com/intent/tweet?text=Wow:&amp;url=https%3A%2F%2Fgithub.com%2Faahnik%2Ftgcf"><img src="https://img.shields.io/twitter/url?style=social&amp;url=https%3A%2F%2Fgithub.com%2Faahnik%2Ftgcf" alt="Twitter"></a></p>\n\n<br>\n\n<!-- markdownlint-enable -->\n\nThe *key features* are:\n\n1. Two **[modes of operation](https://github.com/aahnik/tgcf/wiki/Past-vs-Live-modes-explained)**\nare _past_ or _live_ for dealing with either existing or upcoming messages.\n2. Supports **[login](https://github.com/aahnik/tgcf/wiki/Login-with-a-bot-or-user-account)**\nwith both telegram _bot_ account as well as _user_ account.\n3. Custom **[filter](https://github.com/aahnik/tgcf/wiki/How-to-use-filters-%3F)  [replace](https://github.com/aahnik/tgcf/wiki/Text-Replacement-feature-explained)  [watermark](https://github.com/aahnik/tgcf/wiki/How-to-use--watermarking-%3F)  [ocr](https://github.com/aahnik/tgcf/wiki/You-can-do-OCR-!)** and whatever you need !\n4. Detailed **[docs 📚](https://github.com/aahnik/tgcf/wiki)** +\nVideo tutorial + Help from community in **[discussion forum 😎](https://github.com/aahnik/tgcf/discussions)**.\n5. If you are a python developer, writing **[plugins 🔌](https://github.com/aahnik/tgcf/wiki/How-to-write-a-plugin-for-tgcf-%3F)**\nis like stealing candy from a baby.\n\nWhat are you waiting for? Star 🌟 the repo and click Watch 🕵 to recieve updates.\n\nYou can also join the official [Telegram Channel](https://telegram.me/tg_cf),\nto recieve updates without any ads.\n\n<!-- markdownlint-disable -->\n## Video Tutorial 📺\n\nA youtube video is coming soon. [Subscribe](https://www.youtube.com/channel/UCcEbN0d8iLTB6ZWBE_IDugg) to get notified.\n\n<!-- markdownlint-enable -->\n\n## Run Locally 🔥\n\n> **Note:** Make sure you have Python 3.8 or above installed.\nGo to [python.org](https://python.org) to download python.\n\nClick on your platform of choice for a more detailed guide.\n\n| Platform | Supported |\n| -------- | :-------: |\n| [Windows](https://github.com/aahnik/tgcf/wiki/Run-tgcf-on-Windows)  |     ✅    |\n| Mac      |     ✅     |\n| Linux    |     ✅     |\n| [Android](https://github.com/aahnik/tgcf/wiki/Run-on-Android-using-Termux)  |     ✅     |\n\nIf you are familiar with **Docker**, you may [go that way](https://github.com/aahnik/tgcf/wiki/Install-and-run-using-docker)\nfor an easier life.\n\nOpen your terminal and run the following commands.\n\n```shell\npip install --upgrade tgcf\n```\n\nTo check if the installation succeeded, run\n\n```shell\ntgcf --version\n```\n\nIf you see an error, that means installation failed.\n\n### Configuration 🛠️\n\nConfiguring `tgcf` is easy. You just need two files in your present directory\n(from which tgcf is invoked).\n\n- [`.env`](https://github.com/aahnik/tgcf/wiki/Environment-Variables) : To\ndefine your environment variables easily.\n\n- [`tgcf.config.yml`](https://github.com/aahnik/tgcf/wiki/How-to-configure-tgcf-%3F) :\nAn `yaml` file to configure how `tgcf` behaves.\n\n### Start `tgcf` ✨\n\nIn your terminal, just run `tgcf live` or `tgcf past` to start `tgcf`.\nIt will prompt you to enter your phone no. or bot token, when you run it\nfor the first time.\n\nFor more details run `tgcf --help` or [read docs](https://github.com/aahnik/tgcf/wiki/CLI-Usage).\n\n## Run on Cloud 🌩️\n\nDeploying to a cloud server is an easier alternative if you cannot install\non your own machine.\nCloud servers are very reliable and great for running `tgcf` in live mode\nfor a long time.\n\n<!-- markdownlint-disable -->\n\n| Platform                                                     | Pros                    | Cons                        |\n| ------------------------------------------------------------ | ----------------------- | --------------------------- |\n| <a href="https://github.com/aahnik/tgcf/wiki/Deploy-to-Heroku">   <img src="https://www.herokucdn.com/deploy/button.svg" alt="Deploy to Heroku" width=155></a> | free for 450 hr/mo      | can\'t use tgcf in past mode |\n| <a href="https://github.com/aahnik/tgcf/wiki/Deploy-to-Digital-Ocean">  <img src="https://www.deploytodo.com/do-btn-blue.svg" alt="Deploy to DO" width=220></a> | speed and reliability   | starts from $5/mo           |\n| <a href="https://github.com/aahnik/tgcf/wiki/Run-for-free-on-Gitpod">  <img src="https://gitpod.io/button/open-in-gitpod.svg" alt="Run on Gitpod" width=160></a> | easily edit config file | only 50 hr/mo free          |\n\n\n**Other options**\n\n- [Python Anywhere](https://github.com/aahnik/tgcf/wiki/Run-on-PythonAnywhere)\n- [Google Cloud Run](https://github.com/aahnik/tgcf/wiki/Run-on-Google-Cloud)\n- Scheduled using [GitHub Actions](https://github.com/aahnik/tgcf/wiki/Run-tgcf-in-past-mode-periodically)\n\n\n<!-- markdownlint-enable -->\n\n## Getting Help 💁🏻\n\n- First of all [read the wiki](https://github.com/aahnik/tgcf/wiki)\nand [watch the videos](https://www.youtube.com/channel/UCcEbN0d8iLTB6ZWBE_IDugg)\nto get started.\n- Search your problem everywhere !\n- Feel free to ask your questions in the [Discussion forum](https://github.com/aahnik/tgcf/discussions/new).\n- For reporting bugs or requesting a feature please use the [issue tracker](https://github.com/aahnik/tgcf/issues/new)\nfor this repo.\n\n## Contributing 🙏\n\nPRs most welcome! Read the [contributing guidelines](/.github/CONTRIBUTING.md) to get started.\n\nAlso read:\n\n- [How to write a plugin for tgcf](https://github.com/aahnik/tgcf/wiki/How-to-write-a-plugin-for-tgcf-%3F)\n- [Package management with Poetry](https://python-poetry.org/docs/)\n- [Telethon documentation](https://docs.telethon.dev/en/latest/)\n\nIf you are not a developer, you may also contribute financially to\nincentivise the development of any custom feature you need.\n',
    'author': 'aahnik',
    'author_email': 'daw@aahnik.dev',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/aahnik/tgcf',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""Declare all global constants."""

COMMANDS = {
    "start": "Check whether I am alive",
    "forward": "Set a new forward",
    "remove": "Remove an existing forward",
    "help": "Learn usage",
}

REGISTER_COMMANDS = True

KEEP_LAST_MANY = 10000

CONFIG_FILE_NAME = "tgcf.config.yml"
CONFIG_ENV_VAR_NAME = "TGCF_CONFIG"


class BotMessages:
    """Messages given by the bot to users."""

    # pylint: disable=too-few-public-methods
    start = "Hi! I am alive"
    bot_help = "For details visit github.com/aahnik/tgcf"
